/**
 * $RCSfile: XMLPropertyUtility.java,v $
 * $Revision: 1.1.1.1 $
 * $Date: 2004/01/11 08:46:25 $
 *
 * Copyright (C) 2001-2002 softme.org (Softme Studio). All rights reserved.
 * ===================================================================
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL COOLSERVLETS INC. OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */

package org.softme.utility;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.List;
import java.util.StringTokenizer;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

/**
 * Provides the the ability to use simple XML property files. Each property is
 * in the form X.Y.Z, which would map to an XML snippet of:
 * <pre>
 * <X>
 *     <Y>
 *         <Z>someValue</Z>
 *     </Y>
 * </X>
 * </pre>
 *
 * The XML file is passed in to the constructor and must be readable and
 * writtable. Setting property values will automatically persist those value
 * to disk.
 */
public class XMLPropertyUtility {

    private String file;
    private Document doc;

    /**
     * XMLԴ
     *
     * @parm file ͶȡԵXMLļȫ·.
     */
    public XMLPropertyUtility(String file) {
        this.file = file;
        try {
            SAXBuilder builder = new SAXBuilder();
            doc = builder.build(new File(file));
        }
        catch (Exception e) {
            System.err.println("Error creating XML parser in "
                + "XMLPropertyUtility.java");
            e.printStackTrace();
        }
    }
    
    public String getProperty(String name) {
        String[] propName = parsePropertyName(name);
        // Search for this property by traversing down the XML heirarchy.
        Element element = doc.getRootElement();
        for (int i = 0; i < propName.length; i++) {
            /*element = element.getChild(propName[i]);
            if (element == null) {
                // This node doesn't match this part of the property name which
                // indicates this property doesn't exist so return null.
                return "";
            }*/
                        
            // v1.2.2 жpropName[i]ǷΪ
        	// ֧ prop.is.i.foo ķʽ
        	try{
        		int j = Integer.parseInt(propName[i]); 
        		
        		// propName[i] Ϊ֣ȡΪjֵ
        		List elementList = element.getChildren();
        		element = (Element)elementList.get(j);
        	}
        	catch(NumberFormatException e)
        	{  	
        		element = element.getChild(propName[i]);
        	}
        	
        	if (element == null) {
	            // This node doesn't match this part of the property name which
	            // indicates this property doesn't exist so return null.
	            return null;
	        }	
        }
        // At this point, we found a matching property, so return its value.
        // Empty strings are returned as null.
        String value = element.getText();
        return value.trim();        
    }

    public List getProperties(String name) {
        String[] propName = parsePropertyName(name);
        // Search for this property by traversing down the XML heirarchy.
        Element element = doc.getRootElement();
        for (int i = 0; i < propName.length; i++) {
            element = element.getChild(propName[i]);
            if (element == null) {
                // This node doesn't match this part of the property name which
                // indicates this property doesn't exist so return null.
                return null;
            }
        }
        // At this point, we found a matching property, so return its value.
        // Empty strings are returned as null.
        
        return element.getChildren();
    }

	public void setProperty(String name, String value) {
        String[] propName = parsePropertyName(name);
        // Search for this property by traversing down the XML heirarchy.
        Element element = doc.getRootElement();
        for (int i=0; i<propName.length; i++) {
            // If we don't find this part of the property in the XML heirarchy
            // we add it as a new node
            if (element.getChild(propName[i]) == null) {
                element.addContent(new Element(propName[i]));
            }
            element = element.getChild(propName[i]);
        }
        // Set the value of the property in this node.
        element.setText(value);
        
        // JWikiϵͳУֱдӲ
        // write the XML properties to disk
        //saveProperties();
    }

    public void deleteProperty(String name) {
        String[] propName = parsePropertyName(name);
        // Search for this property by traversing down the XML heirarchy.
        Element element = doc.getRootElement();
        for (int i=0; i<propName.length-1; i++) {
            element = element.getChild(propName[i]);
            // Can't find the property so return.
            if (element == null) {
                return;
            }
        }
        // Found the correct element to remove, so remove it...
        element.removeChild(propName[propName.length-1]);
        // .. then write to disk.
        saveProperties();
    }

    /**
     * Saves the properties to disk as an XML document named webpm_config.xml
     * under your webpmHome/conf directory.
     */
    private synchronized void saveProperties() {
        OutputStream out = null;
        try {
            // Use JDOM's XMLOutputter to do the writing and formatting.
            XMLOutputter outputter = new XMLOutputter();
            out = new BufferedOutputStream(new FileOutputStream(new File(file)));
            outputter.output(doc, out);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        finally {
            try {  out.close();  }
            catch (Exception e) { }
        }
    }

    /**
     * Returns an array representation of the given WebPM property. WebPM
     * properties are always in the format "prop.name.is.this" which would be
     * represented as an array of four Strings.
     *
     * @param name the name of the WebPM property.
     * @return an array representation of the given WebPM property.
     */
    private String[] parsePropertyName(String name) {
        // Figure out the number of parts of the name (this becomes the size
        // of the resulting array).
        int size = 1;
        for (int i=0; i<name.length(); i++) {
            if (name.charAt(i) == '.') {
                size++;
            }
        }
        String[] propName = new String[size];
        // Use a StringTokenizer to tokenize the property name.
        StringTokenizer tokenizer = new StringTokenizer(name, ".");
        int i = 0;
        while (tokenizer.hasMoreTokens()) {
            propName[i] = tokenizer.nextToken();
            i++;
        }
        return propName;
    }
} 