package org.softme.webpm.info.project;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;

import org.softme.webpm.pub.DBConnection;
import org.softme.webpm.pub.WebPMConfig;
import org.softme.webpm.utility.StringUtility;

/**
 * <p> Ŀʵ
 * Copyleft (l) 2001-2002 Softme Studio
 * @author BrokenDoor
 * @version 1.0
 */

public class ProjectFactoryImp extends ProjectFactory {
	
	public Project getProject(int pID) throws ProjectNotFoundException {
		// 
		if (pID<=0) {
			return null;
		}
		String GET_PROJECT_BY_ID = "SELECT * FROM t_webpm_project where fid=?";
		Connection conn = null;
        PreparedStatement pstmt = null;
        Project existingProject = null;
        try{
			conn = DBConnection.newInstance() ;
            pstmt = conn.prepareStatement(GET_PROJECT_BY_ID) ;
        	pstmt.setInt(1,pID) ;
            ResultSet arRs = pstmt.executeQuery() ;
            if(arRs.next()){
                existingProject = this.loadProjectFromResultSet(arRs) ;
            }else{
                throw new ProjectNotFoundException("ProjectFactoryError: project not found..");
            }
        }catch(SQLException e){
        	WebPMConfig.logger.error(e);
//            e.printStackTrace() ;
            return null;
        }finally{
            DBConnection.closeConnection(pstmt,conn) ;
        }
        return existingProject;
	}
	
	public Project getProject(String pName) throws ProjectNotFoundException{
		// 
		if (pName==null||pName.length()<0) {
			return null;
		}
		String GET_PROJECT_BY_NAME = "SELECT * FROM t_webpm_project where fprojectname=?";
		Connection conn = null;
        PreparedStatement pstmt = null;
        Project prj = null;
        try{
			conn = DBConnection.newInstance() ;
            pstmt = conn.prepareStatement(GET_PROJECT_BY_NAME) ;
        	pstmt.setString(1,StringUtility.encodeMemory2DB(pName)) ;
            ResultSet arRs = pstmt.executeQuery() ;
            if(arRs.next()){
                prj = this.loadProjectFromResultSet(arRs) ;
            }else{
                throw new ProjectNotFoundException("ProjectFactoryError: project not found..");
            }
        }catch(SQLException e){
        	WebPMConfig.logger.error(e);
//            e.printStackTrace() ;
            return null;
        }finally{
            DBConnection.closeConnection(pstmt,conn) ;
        }
        return prj;	
	}
	
	/* Create New Project
	*/
	public Project createProject(){
		return new ProjectImp();	
	}
	
	public Project createProject (String pPrjName, 
			String pPrjAuthor,
			String pPrjKeywords,
			String pPrjDesc) 
			throws ProjectAlreadyExistsException {
		Project newProject = null;
		try{
			newProject = this.getProject(pPrjName);
			throw new ProjectAlreadyExistsException();
		}
		catch(ProjectNotFoundException e){
			newProject = this.createProject();
			newProject.setProjectName(pPrjName);
			newProject.setAuthor(pPrjAuthor);
			newProject.setKeywords(pPrjKeywords);
			newProject.setDescription(pPrjDesc);
			newProject.save();
		}
		return newProject;	
	}
	
	/**
	 * <p> ݿȡĿ
	 * @param pRs 
	 */
	 public Project loadProjectFromResultSet(ResultSet pRs){
		Project prj = this.createProject();
		try{
			prj.setProjectID(pRs.getInt("fid"));
	        prj.setProjectName(StringUtility.encodeDB2Memory(pRs.getString("fprojectname"))) ;
	        prj.setKeywords(StringUtility.encodeDB2Memory(pRs.getString("fkeywords"))) ;
	        prj.setDescription(StringUtility.encodeDB2Memory(pRs.getString("fdescription"))) ;
	        prj.setAuthor(StringUtility.encodeDB2Memory(pRs.getString("fauthor")));
	        prj.setCreateDate(pRs.getString("fcreatedate"));
	        prj.setUpdateDate(pRs.getString("fupdatedate"));
	        prj.setFlag(pRs.getInt("fflag")) ;
		}catch(SQLException e){
            WebPMConfig.logger.error(e);
            return null;
        }
		return prj;
	}
	
	public Iterator getAllProjects(){
		String GET_ALL_PROJECT="SELECT * FROM t_webpm_project ORDER BY fid desc";
      	Connection conn=null;
      	PreparedStatement pstmt=null;
      	Project prj=null;
      	ArrayList prjList=new ArrayList();
      	try{
          	conn=DBConnection.newInstance() ;
          	pstmt=conn.prepareStatement(GET_ALL_PROJECT) ;   
          	ResultSet arRs=pstmt.executeQuery() ;
          	while(arRs.next()){
              	prj = this.loadProjectFromResultSet(arRs) ;
              	prjList.add(prj) ;
          	}
      	}catch(SQLException e){
          	WebPMConfig.logger.error(e);
      	}finally{
          	DBConnection.closeConnection(pstmt,conn) ;
      	}
      	
      	return prjList.iterator() ;
	}
}
