package org.softme.webpm.jwiki.article;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;

import org.softme.webpm.jwiki.KeyIndex;
import org.softme.webpm.jwiki.exception.ArticleAlreadyExistsException;
import org.softme.webpm.jwiki.exception.ArticleAlreadyModifyException;
import org.softme.webpm.jwiki.exception.ArticleNotFoundException;
import org.softme.webpm.jwiki.exception.BodyNotChangeException;
import org.softme.webpm.jwiki.query.Query;
import org.softme.webpm.jwiki.query.QueryImp;
import org.softme.webpm.pub.DBConnection;
import org.softme.webpm.pub.Debug;
import org.softme.webpm.pub.WebPMConfig;
import org.softme.webpm.utility.StringUtility;


/**
 * Title:        WebPM
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      WebPM
 * @author 
 * @version 1.0
 */

public class ArticleFactoryImp extends ArticleFactory {

    public Article newArticle(){
        return new ArticleImp();
    }

    public Article getArticle(String pTopic) throws ArticleNotFoundException{
        pTopic=StringUtility.encodeMemory2DB(pTopic);
        final String GET_ONE_ARTICLE="SELECT * FROM t_wiki_article WHERE ftopic='"+pTopic+"'";
        return this.getArticleFromDB(GET_ONE_ARTICLE) ;
    }
    public Article getArticle(int pID) throws ArticleNotFoundException{
        final String GET_ONE_ARTICLE="SELECT * FROM t_wiki_article WHERE fid="+pID;
        return this.getArticleFromDB(GET_ONE_ARTICLE) ;
    }

    public Iterator getArticleByUser(String pUserName){
        final String GET_USER_ARTICLE="SELECT * FROM t_wiki_article WHERE fauthor=? order by fcreatedate desc";
        Connection conn=null;
        PreparedStatement pstmt=null;
        ArrayList arList=new ArrayList();
        try{
            conn=DBConnection.newInstance() ;
            pstmt=conn.prepareStatement(GET_USER_ARTICLE) ;
            pstmt.setString(1,StringUtility.encodeMemory2DB(pUserName)) ;
            ResultSet arRs=pstmt.executeQuery() ;
            Article userAr=null;
            while(arRs.next()){
                userAr=this.loadArticleFromResultSet(arRs) ;
                arList.add(userAr) ;
            }

        }catch(SQLException e){
            //e.printStackTrace();
            WebPMConfig.logger.error(e);
            

        }finally{
            DBConnection.closeConnection(pstmt,conn) ;
        }
        return arList.iterator() ;
    }
    public Article getVersion(int pID) throws ArticleNotFoundException{
        String GET_ONE_BACKUP="SELECT * FROM t_wiki_articlebackup WHERE fid="+pID;
        return this.getArticleFromDB(GET_ONE_BACKUP) ;
    }
    public Article createArticle(String pTopic, String pBody, String pAuthor) throws  ArticleAlreadyExistsException {
        KeyIndex keyIndex=KeyIndex.getInstance() ;

        keyIndex.addKey(pTopic) ;
        try{
            getArticle(pTopic);
            throw new ArticleAlreadyExistsException();
        }catch(ArticleNotFoundException e){
            return new ArticleImp(pTopic,pBody,pAuthor);
        }

    }
    public Iterator getRecentChanges(){
        String GET_RECENT="select *  from t_wiki_article  WHERE TO_DAYS(NOW()) - TO_DAYS(fupdatedate) <=10 order by fupdatedate desc";
        Connection conn=null;
        PreparedStatement pstmt=null;
        Article recentAr=null;
        ArrayList recentList=new ArrayList();
        try{
            conn=DBConnection.newInstance() ;
            pstmt=conn.prepareStatement(GET_RECENT) ;
            ResultSet arRs=pstmt.executeQuery() ;
            while(arRs.next()){
                recentAr=this.loadArticleFromResultSet(arRs) ;
                recentList.add(recentAr) ;
            }
        }catch(SQLException e){
            Debug.println(e.toString()) ;
            e.printStackTrace() ;
        }finally{
            DBConnection.closeConnection(pstmt,conn) ;
        }
        return recentList.iterator() ;
    }

    public Query createQuery(String pKey) {
        return new QueryImp(pKey);
    }

    public Iterator getVersions(String pTopic){
      String GET_ALL_BACKUP="SELECT * FROM t_wiki_articlebackup WHERE ftopic=? order by fid desc";
      Connection conn=null;
      PreparedStatement pstmt=null;
      Article backupA=null;
      ArrayList backupList=new ArrayList();
      try{
          conn=DBConnection.newInstance() ;
          pstmt=conn.prepareStatement(GET_ALL_BACKUP) ;
          pTopic=StringUtility.encodeMemory2DB(pTopic) ;
          pstmt.setString(1,pTopic) ;
          ResultSet arRs=pstmt.executeQuery() ;
          while(arRs.next()){
              backupA=this.loadArticleFromResultSet(arRs) ;
              backupList.add(backupA) ;
          }
      }catch(SQLException e){
          e.printStackTrace();
          Debug.println(e.toString()) ;
      }finally{
          DBConnection.closeConnection(pstmt,conn) ;
      }
      return backupList.iterator() ;
     }

    public Iterator getRecentVisit(){
        String GET_RECENT_PAGE="SELECT * FROM t_wiki_visitor order by fvisitdate desc";
        return this.loadVisitInfo(GET_RECENT_PAGE);
    }
    public Iterator getActivePage(){

        String GET_ACTIVE_PAGE="SELECT * FROM t_wiki_visitor  order by fvisittimes desc";
        return this.loadVisitInfo(GET_ACTIVE_PAGE) ;
    }

    public void updateArticleBody(Article pNewArticle,String pLastUpdateDate) throws ArticleAlreadyModifyException,BodyNotChangeException,ArticleNotFoundException{

            Article oldArticle=getArticle(pNewArticle.getTopic());
            pNewArticle.setCreateDate(oldArticle.getCreateDate()) ;//Ҫʱ.

            if(!oldArticle.getCreateDate().equals(pLastUpdateDate)){

                throw new ArticleAlreadyModifyException();

            }else if(oldArticle.getBody().equals(pNewArticle.getBody())){

                throw new BodyNotChangeException();

            }else{

                pNewArticle.modifyBody() ;

            }

    }

    public Article revert(int pBackupID) {
        Article backup=null,ar=null;
        try{
            backup=getVersion(pBackupID);
            ar=getArticle(backup.getTopic());
            ar.setBody(backup.getBody()) ;
            ar.setAuthor(backup.getAuthor()) ;
            ar.setCreateDate(backup.getCreateDate());
            //ݻָʱΪʱ
            //ar.setUpdateDate(backup.getUpdateDate()) ;
            ar.setFlag(backup.getFlag()) ;
            ar.setMender(backup.getMender()) ;
            //ar.modifyBody() ;//¸»Զһ
            ar.updateAllInfo() ;

        }catch(Exception e){

            e.printStackTrace() ;
        }
        return backup;
    }
    private Iterator loadVisitInfo(String pSql){
        int MAX_PAGE_NUMBER=20;
        //String GET_ACTIVE_PAGE="SELECT * FROM t_wiki_visitor  order by fvisittimes desc";
        Connection conn=null;
        PreparedStatement pstmt=null;
        ArrayList pageList=new ArrayList();
        try{
            conn=DBConnection.newInstance() ;
            pstmt=conn.prepareStatement(pSql) ;
            ResultSet arRs=pstmt.executeQuery() ;
            int pageNumber=0;
            VisitInfo visitInfo=null;
            while(arRs.next()){
                pageNumber++;
                if(pageNumber>MAX_PAGE_NUMBER){
                    break;
                }
                visitInfo=new VisitInfo(StringUtility.encodeDB2Memory(arRs.getString("ftopic")),
                                        StringUtility.encodeDB2Memory(arRs.getString("fvisitor")),
                                        arRs.getString("fvisitdate"),
                                        arRs.getInt("fvisittimes"));
                pageList.add(visitInfo) ;


            }

        }catch(SQLException e){
            e.printStackTrace();
            Debug.println(e.toString()) ;

        }finally{
            DBConnection.closeConnection(pstmt,conn) ;
        }
        return pageList.iterator() ;
    }
    private Article getArticleFromDB(String pSQL) throws ArticleNotFoundException{
        Connection conn=null;
        PreparedStatement pstmt=null;
        Article existingAr=null;
        try{
            conn=DBConnection.newInstance() ;
            pstmt=conn.prepareStatement(pSQL) ;
            ResultSet arRs=pstmt.executeQuery() ;
            if(arRs.next()){
                existingAr=this.loadArticleFromResultSet(arRs) ;
            }else{
                throw new ArticleNotFoundException("ArticleFactoryError: article not found..");
            }
        }catch(SQLException e){
            Debug.println(e.toString()) ;
            e.printStackTrace() ;
            return null;
        }finally{
            DBConnection.closeConnection(pstmt,conn) ;
        }
        return existingAr;
    }
    private Article loadArticleFromResultSet(ResultSet pRs) throws SQLException{
        Article existingAr= newArticle();
        if(pRs==null) return null;
        
        existingAr.setID(pRs.getInt("fid")) ;
        existingAr.setTopic(StringUtility.encodeDB2Memory(pRs.getString("ftopic")));
        existingAr.setBody(StringUtility.encodeDB2Memory(pRs.getString("fbody"))) ;
        existingAr.setCreateDate(pRs.getString("fcreatedate")) ;
        existingAr.setUpdateDate(pRs.getString("fupdatedate")) ;
        existingAr.setFlag(pRs.getInt("fflag")) ;
        existingAr.setMender(StringUtility.encodeDB2Memory(pRs.getString("fmender"))) ;
        existingAr.setAuthor(StringUtility.encodeDB2Memory(pRs.getString("fauthor"))) ;
        return existingAr;
    }

}