package org.softme.webpm.test.info.project;

import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.softme.webpm.info.project.MemberNotFoundInProjectException;
import org.softme.webpm.info.project.Project;
import org.softme.webpm.info.project.ProjectAlreadyExistsException;
import org.softme.webpm.info.project.ProjectFactory;
import org.softme.webpm.info.project.ProjectMember;
import org.softme.webpm.info.project.ProjectMemberAlreadyExistsException;
import org.softme.webpm.info.project.ProjectNotFoundException;
import org.softme.webpm.info.user.UserNotFoundException;

/**
 * Title:        Project Test
 * Description:
 * Copyright:    Copyleft (l) 2002
 * Company:      Softme Studio
 * @author BrokenDoor
 * @version 1.0
 */

public class ProjectTest extends TestCase{
	private String gProjectName = "test";
	private String gProjectAuthor = "test";
	private String gProjectDescMod = "test project1";
	private String gProjectDesc = "Test Project";
	private String gProjectKeywords = "Test Keywords";
	
	private String gProjectMember = "brokendoor";
	
	private ProjectFactory prjFactory = ProjectFactory.getInstance();
	
	public ProjectTest(String name) {
		super(name);
	}
	
	public static Test suite(){
		return new TestSuite(ProjectTest.class);
	}
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(suite());
	}	
	
	public void testCreateProject(){
		try{
			Project newProject = prjFactory.createProject(
					gProjectName, 
					gProjectAuthor,
					gProjectKeywords,
					gProjectDesc);			
		} catch(ProjectAlreadyExistsException e){
			// ĿѾڣͨ			
		} catch(Exception e){
			fail(e.getMessage());
		}
	}
	
	public void testGetProject(){
		try{
			Project existingProject = prjFactory.getProject(gProjectName);
			assertEquals(existingProject.getProjectName(),gProjectName);
			//existingProject = prjFactory.getProject(4);
			//assertEquals(existingProject.getProjectName(),gProjectName);
		} catch(ProjectNotFoundException e) {
			fail(e.getMessage()); //ڣʧ
		}
	}
	
	// test for save project
	public void testSaveProject(){
		try{
			Project	prj = prjFactory.getProject(gProjectName);
			prj.setDescription(gProjectDescMod);
			prj.save();
			prj = prjFactory.getProject(gProjectName);
			assertEquals(prj.getDescription(), gProjectDescMod);
			prj.setDescription(gProjectDesc);
			prj.save();		
		} catch (ProjectNotFoundException e) {
			fail(e.getMessage()); //ڣʧ
		}		
	}
	
	// test for project list
	public void testGetAllProjects() {
		Iterator projects = prjFactory.getAllProjects();
		Project prj = null;
		while (projects.hasNext()) {
			prj = (Project) projects.next();	
			assertNotNull(prj.getProjectName());
			assertNotNull(prj.getKeywords());
		}	
	}
	
	// test for add project member
	public void testAddMember() {		
		try{
			Project prj = prjFactory.getProject(gProjectName);			
			assertTrue(prj.addMember(gProjectMember));	
		} catch(ProjectNotFoundException e) {
			fail(e.getMessage());			
		} catch(UserNotFoundException e) {
			fail(e.getMessage());
		} catch(ProjectMemberAlreadyExistsException e) {
			// Ѿڣͨ	
		}
	}
	
	// test for get project member
	public void testGetMember() {
		try {
			Project prj = prjFactory.getProject(gProjectName);	
			ProjectMember user = prj.getMember(gProjectMember);
			assertEquals(user.getUserName(),gProjectMember);			
		} catch(ProjectNotFoundException e) {
			fail(e.getMessage());
		} catch(MemberNotFoundInProjectException e) {
			fail(e.getMessage());
		} catch(UserNotFoundException e) {
			fail(e.getMessage());
		} 
	}
	
	// test for remove project member 
	public void testRemoveMember(){
		try{
			Project prj = prjFactory.getProject(gProjectName);
			assertTrue(prj.removeMember(gProjectMember));			
		} catch(UserNotFoundException e) {
			fail(e.getMessage());
		} catch(ProjectNotFoundException e) {
			fail(e.getMessage());
		} catch(MemberNotFoundInProjectException e) {
			fail(e.getMessage());
		} 
	}
	
	// test for get project member lists
	public void testGetAllMembers(){
		try{
			Project prj = prjFactory.getProject(gProjectName);
			Iterator members = prj.getAllMembers();
			assertTrue(members.hasNext());
		} catch(Exception e){
			fail(e.getMessage());
		}
			
	}
}